@@ -24,19 +24,24 @@
 #include "includes.h"
 #include "system/filesys.h"
 #include "system/locale.h"
+#if HAVE_SYSTEMD
+#include <systemd/sd-daemon.h>
+#endif
 
 /*******************************************************************
  Close the low 3 fd's and open dev/null in their place.
 ********************************************************************/
 
-_PUBLIC_ void close_low_fds(bool stderr_too)
+_PUBLIC_ void close_low_fds(bool stdin_too, bool stdout_too, bool stderr_too)
 {
 #ifndef VALGRIND
 	int fd;
 	int i;
 
-	close(0);
-	close(1);
+	if (stdin_too)
+		close(0);
+	if (stdout_too)
+		close(1);
 
 	if (stderr_too)
 		close(2);
@@ -44,6 +49,10 @@ _PUBLIC_ void close_low_fds(bool stderr_too)
 	/* try and use up these file descriptors, so silly
 		library routines writing to stdout etc won't cause havoc */
 	for (i=0;i<3;i++) {
+		if (i == 0 && !stdin_too)
+			continue;
+		if (i == 1 && !stdout_too)
+			continue;
 		if (i == 2 && !stderr_too)
 			continue;
 
@@ -56,6 +65,7 @@ _PUBLIC_ void close_low_fds(bool stderr_too)
 		}
 		if (fd != i) {
 			DEBUG(0,("Didn't get file descriptor %d\n",i));
+			close(fd);
 			return;
 		}
 	}
@@ -68,8 +78,13 @@ _PUBLIC_ void close_low_fds(bool stderr_too)
 
 _PUBLIC_ void become_daemon(bool do_fork, bool no_process_group, bool log_stdout)
 {
+	pid_t newpid;
 	if (do_fork) {
-		if (sys_fork()) {
+		newpid = fork();
+		if (newpid) {
+#if HAVE_SYSTEMD
+			sd_notifyf(0, "READY=0\nSTATUS=Starting process...\nMAINPID=%lu", (unsigned long) newpid);
+#endif /* HAVE_SYSTEMD */
 			_exit(0);
 		}
 	}
@@ -79,7 +94,7 @@ _PUBLIC_ void become_daemon(bool do_fork, bool no_process_group, bool log_stdout
 	if (!no_process_group) setsid();
 #elif defined(TIOCNOTTY)
 	if (!no_process_group) {
-		int i = sys_open("/dev/tty", O_RDWR, 0);
+		int i = open("/dev/tty", O_RDWR, 0);
 		if (i != -1) {
 			ioctl(i, (int) TIOCNOTTY, (char *)0);
 			close(i);
@@ -87,9 +102,36 @@ _PUBLIC_ void become_daemon(bool do_fork, bool no_process_group, bool log_stdout
 	}
 #endif /* HAVE_SETSID */
 
-	if (!log_stdout) {
-		/* Close fd's 0,1,2. Needed if started by rsh */
-		close_low_fds(false);  /* Don't close stderr, let the debug system
-					  attach it to the logfile */
+	/* Close fd's 0,1,2 as appropriate. Needed if started by rsh. */
+	/* stdin must be open if we do not fork, for monitoring for
+	 * close.  stdout must be open if we are logging there, and we
+	 * never close stderr (but debug might dup it onto a log file) */
+	close_low_fds(do_fork, !log_stdout, false);
+}
+
+_PUBLIC_ void exit_daemon(const char *msg, int error)
+{
+#ifdef HAVE_SYSTEMD
+	if (msg == NULL) {
+		msg = strerror(error);
 	}
+
+	sd_notifyf(0, "STATUS=daemon failed to start: %s\n"
+				  "ERRNO=%i",
+				  msg,
+				  error);
+#endif
+	DEBUG(0, ("STATUS=daemon failed to start: %s, error code %d\n", msg, error));
+	exit(1);
+}
+
+_PUBLIC_ void daemon_ready(const char *daemon)
+{
+	if (daemon == NULL) {
+		daemon = "Samba";
+	}
+#ifdef HAVE_SYSTEMD
+	sd_notifyf(0, "READY=1\nSTATUS=%s: ready to serve connections...", daemon);
+#endif
+	DEBUG(0, ("STATUS=daemon '%s' finished starting up and ready to serve connections", daemon));
 }
